library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library work;
use work.PkgSdramController.all;
use work.PkgPsm.all;

entity FreePtrCache is
port (
       --Packet reader Interface
       FreePtrIn           : in std_logic_vector(c_ChunkPointerWidth-1 downto 0);
       FreePtrValidIn      : in std_logic;
       WriteError          : out std_logic;

       --Chunk Writer Interface
       GetFreePtr          : in std_logic;
       FreePtrValidOut     : out std_logic;
       BufferFull          : out std_logic;
       FreePtrOut          : out std_logic_vector(c_ChunkPointerWidth-1 downto 0);

       SdramFreePtrs       : inout std_logic_vector(c_DataWidth-1 downto 0);
       SdramFreePtrFifoAddr: out std_logic_vector(c_AddrWidth-1 downto 0);
       SdramFreePtrOp      : out std_logic;
       SdramFreePtrOpValid : out std_logic;
       SdramFreePtrGrant   : in std_logic;
       SdramFreePtrPullReq : in std_logic;

       Clk                 : in std_logic;
       Reset               : in std_logic);
end FreePtrCache;


architecture behave of FreePtrCache is

signal    addra: std_logic_VECTOR(7 downto 0);
signal    douta: std_logic_VECTOR(63 downto 0);
signal    addrb: std_logic_VECTOR(7 downto 0);
signal    dinb: std_logic_VECTOR(63 downto 0);
signal    doutb: std_logic_VECTOR(63 downto 0);
signal    web: std_logic;

component freeptrs
        port (
        clka: IN std_logic;
        addra: IN std_logic_VECTOR(7 downto 0);
        douta: OUT std_logic_VECTOR(63 downto 0);
        clkb: IN std_logic;
        addrb: IN std_logic_VECTOR(7 downto 0);
        dinb: IN std_logic_VECTOR(63 downto 0);
        doutb: OUT std_logic_VECTOR(63 downto 0);
        web: IN std_logic);
end component;

-- Synplicity black box declaration
attribute syn_black_box : boolean;
attribute syn_black_box of freeptrs: component is true;


signal WrMswLsw : std_logic; -- 0 = Msw, 1 = Lsw
signal RdMswLsw : std_logic; -- 0 = Msw, 1 = Lsw
signal PtrCount : integer range 0 to 512;
signal ClkGetFreePtr : std_logic;
signal TmpCount : integer range 0 to 600;
signal Initialize : std_logic;
signal Threshold1 : integer;
signal Threshold2 : integer;
signal SdramFifoReadPointer : std_logic_vector(c_AddrWidth-1 downto 0);
signal SdramFifoWritePointer: std_logic_vector(c_AddrWidth-1 downto 0);
signal ClkTakePtrs : std_logic;
signal TakePtrs : std_logic;
signal ClkSdramFreePtrGrant : std_logic; 
begin

myfreeptrs : freeptrs
        port map (
            clka => Clk,
            addra => addra,
            douta => douta,
            clkb => Clk,
            addrb => addrb,
            dinb => dinb,
            doutb => doutb,
            web => web);

process(Clk)

variable v_PtrCount : integer range 0 to 512;
begin

if Clk = '1' and Clk'event then
  if Reset = '0' then
     WrMswLsw <= '0';
     RdMswLsw <= '0';

     addrb <= (others => '1');
     dinb  <= (others => '0');
     web   <= '0';

     addra <= (others => '0');

     WriteError <= '0';

     FreePtrValidOut <= '0';
     BufferFull <= '0';
     FreePtrOut <= (others => '0');

     PtrCount   <= 0;
     v_PtrCount := 0;

     ClkGetFreePtr <= '0';
     TmpCount  <= 32;
     Initialize <= '1';

     SdramFreePtrs       <= (others => 'Z');
     SdramFreePtrFifoAddr<= (others => '0');
     SdramFreePtrOp      <= '0';
     SdramFreePtrOpValid <= '0';

     Threshold1 <= 16; 
     Threshold2 <= 248; 

     SdramFifoReadPointer <= c_SdramFifoStartMarker; 
     SdramFifoWritePointer<= c_SdramFifoStartMarker; 

  else


    --Initialize the Fifo
    if Initialize = '1' then
       if web = '1' and addrb = "11111111" then
          Initialize <= '0';
          web <= '0';
       else
          PtrCount <= PtrCount + 2;
          web <= '1';
          addrb <= addrb + '1';
          dinb(63 downto 32) <= conv_std_logic_vector(TmpCount,32);
          dinb(31 downto 0) <= conv_std_logic_vector(TmpCount + 1,32);
          TmpCount <= TmpCount + 2;
       end if;
    end if;



     v_PtrCount := 0;
     if Initialize = '0' then
        v_PtrCount := PtrCount;
        --Putting the Ptr into the cache
        web <= '0';
        if FreePtrValidIn = '1' then
           if WrMswLsw = '0' then
              dinb(63 downto 32) <= FreePtrIn;
              WrMswLsw <= '1';
           else
              dinb(31 downto 0) <= FreePtrIn;
              WrMswLsw <= '0';
              web <= '1';
              addrb <= addrb + '1';
              v_PtrCount := v_PtrCount + 2;
           end if;
        end if;
           
   
        --Pulling the free ptr from the cache
        FreePtrValidOut <= '0';
        if GetFreePtr = '1' and ClkGetFreePtr = '0' then  -- if a pull request
           if v_PtrCount /= 0 then
              if RdMswLsw = '0' then
                 FreePtrOut <= douta(63 downto 32);
                 FreePtrValidOut <= '1';
                 RdMswLsw <= '1';
               else
                 FreePtrOut <= douta(31 downto 0);
                 FreePtrValidOut <= '1';
                 RdMswLsw <= '0';
                 v_PtrCount := v_PtrCount - 2;
                 addra <= addra + '1';
               end if;
            end if;
         end if;
         ClkGetFreePtr <= GetFreePtr;
     end if; --if Initialize = 0

-------------------------------------------------------------------------------------
--Sdram section
-------------------------------------------------------------------------------------

     --Initialization of the SDRAM FIFO

     --Getting Free Pointers from the SDRAM
     if v_PtrCount < Threshold1 and RequestSent = '0' then
        SdramFreePtrFifoAddr <= SdramFifoReadPointer;
        SdramFreePtrOp       <= '0'; 
        SdramOpType          <= '0'; 
        SdramFreePtrOpValid  <= '1'; 
     end if;

     if SdramFreePtrPullReq = '1' and SdramOpType = '0' then
        SdramFreePtrFifoAddr <= (others => '0');
        SdramFreePtrOp       <= '0'; 
        SdramFreePtrOpValid  <= '0'; 
        RequestSent <= '1';
        if SdramFifoReadPointer + "1000" > c_SdramFifoEndMarker then
           SdramFifoReadPointer <= SdramFifoReadPointer + "111" - c_SdramFifoEndMarker + c_SdramFifoStartMarker;
        else
           SdramFifoReadPointer <= SdramFifoReadPointer + "1000";
        end if;
     end if;

     if ClkSdramFreePtrGrant = '1'  and SdramOpType = '0' then  
        web  <= '1';
        dinb <= SdramFreePtrs;  
        SdramTransferCntr <= 0;
     end if;
     if web = '1' then
        addrb <= addrb + '1';
        if SdramTransferCntr /= 7 then
           web  <= '1';
           dinb <= SdramFreePtrs;
           SdramTransferCntr <= SdramTransferCntr + 1;
           v_PtrCount := v_PtrCount + 2;
        else
           web <= '0';
           RequestSent <= '0';
        end if;
     end if;
    
     --Putting the Free Pointers into the SDRAM
     if v_PtrCount > Threshold2 and RequestSent = '0' then
        SdramFreePtrFifoAddr <= SdramFifoWritePointer;
        SdramFreePtrOp       <= '1';
        SdramOpType          <= '1'; 
        SdramFreePtrOpValid  <= '1';
     end if;

     if SdramFreePtrPullReq = '1' and SdramOpType = '1' then
        SdramFreePtrFifoAddr <= (others => '0');
        SdramFreePtrOp       <= '0';
        SdramFreePtrOpValid  <= '0';
        RequestSent <= '1';
        if SdramFifoWritePointer + "1000" > c_SdramFifoEndMarker then
           SdramFifoWritePointer <= SdramFifoWritePointer + "111" - c_SdramFifoEndMarker + c_SdramFifoStartMarker;
        else
           SdramFifoWritePointer <= SdramFifoWritePointer + "1000";
        end if;
     end if;

     if SdramFreePtrGrant = '1'  and SdramOpType = '1' then
        web  <= '0';
        TakePtrs <= '1';
        SdramTransferCntr <= 0;
     end if;
     if TakePtrs = '1' then
        addrb <= addrb - '1';
        if SdramTransferCntr /= 7 then
           TakePtrs  <= '1';
           dinb <= SdramFreePtrs;
           SdramTransferCntr <= SdramTransferCntr + 1;
           v_PtrCount := v_PtrCount + 2;
        else
           TakePtrs <= '0';
           RequestSent <= '0';
        end if;
     end if;
     ClkTakePtrs <= TakePts;
     if ClkTakePtrs = '1' then
        SdramFreePtrs <= doutb; 
     end if;

        

  PtrCount <= v_PtrCount;
  end if; --if reset = 0
end if; --if Clk = 1

end process;
end behave;
