LIBRARY ieee;
LIBRARY work;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;
USE ieee.std_logic_unsigned.ALL;
use work.PkgSdramController.all;
use work.PkgPsm.all;

entity FreePtrManager is
port (

       Initialization      : out std_logic;
       --Packet reader Interface
       FreePtrIn           : in std_logic_vector(c_ChunkPointerWidth-1 downto 0);
       FreePtrValidIn      : in std_logic;
       WriteError          : out std_logic;

       --Chunk Writer Interface
       GetFreePtr          : in std_logic;
       FreePtrValidOut     : out std_logic;
       BufferFull          : out std_logic;
       FreePtrOut          : out std_logic_vector(c_ChunkPointerWidth-1 downto 0);

       --Request Sequencer Interface
       SdramFreePtrs       : inout std_logic_vector(c_DataWidth-1 downto 0);
       SdramFreePtrFifoAddr: out std_logic_vector(c_AddressWidth-1 downto 0);
       SdramFreePtrOp      : out std_logic;
       SdramFreePtrOpValid : out std_logic;
       SdramFreePtrGrant   : in std_logic;
       SdramFreePtrPullReq : in std_logic;

       Clk                 : in std_logic;
       Reset               : in std_logic);
end FreePtrManager;


architecture behave of FreePtrManager is

signal ClkReset : std_logic;

signal    addra: std_logic_VECTOR(7 downto 0);
signal    douta: std_logic_VECTOR(63 downto 0);
signal    addrb: std_logic_VECTOR(7 downto 0);
signal    dinb: std_logic_VECTOR(63 downto 0);
signal    doutb: std_logic_VECTOR(63 downto 0);
signal    web: std_logic;
signal    BufferFull1          : std_logic;

component freeptrs
        port (
        clka: IN std_logic;
        addra: IN std_logic_VECTOR(7 downto 0);
        douta: OUT std_logic_VECTOR(63 downto 0);
        clkb: IN std_logic;
        addrb: IN std_logic_VECTOR(7 downto 0);
        dinb: IN std_logic_VECTOR(63 downto 0);
        doutb: OUT std_logic_VECTOR(63 downto 0);
        web: IN std_logic);
end component;

-- Synplicity black box declaration
attribute syn_black_box : boolean;
attribute syn_black_box of freeptrs: component is true;


signal CachePtrCount : integer range 0 to 512;
signal TmpCount : integer range 0 to 600;

signal SdramFifoReadPointer : std_logic_vector(c_AddressWidth-1 downto 0);
signal SdramFifoWritePointer: std_logic_vector(c_AddressWidth-1 downto 0);
signal SdramFifoPtrCount : integer range 0 to c_EffectiveChunks;
signal InitializeFifoCntr  : std_logic_vector(c_ChunkPointerWidth-1 downto 0);
signal NumRequestsSent : integer range 0 to c_ChunksReserved;
signal InitializeBram : std_logic;
signal ClkFreePtrIn : std_logic_vector(c_ChunkPointerWidth-1 downto 0);
type t_SdramRequestStatesInit is (e_StartInit, e_RequestInit, e_WaitAckInit, e_GotAckInit, e_EndInit);
signal PresentSdramRequestStateInit, NextSdramRequestStateInit  : t_SdramRequestStatesInit;

type t_SdramTransferStatesInit is (e_StartInit, e_WaitGrantInit, e_Write0Init, e_Write1Init, e_Write2Init, e_Write3Init, 
                                   e_Write4Init, e_Write5Init, e_Write6Init, e_Write7Init, e_EndInit); 
signal PresentSdramTransferStateInit, NextSdramTransferStateInit: t_SdramTransferStatesInit;

type t_SdramTransferStates is (e_Start, e_Check, e_RequestRead, e_RequestWrite, e_WaitAckRead, e_WaitAckWrite,
                               e_WaitGrantRead, e_WaitGrantWrite, 
                               e_Write0, e_Write1, e_Write2, e_Write3, e_Write4, e_Write5, e_Write6, e_Write7, 
                               e_Read0, e_Read1, e_Read2, e_Read3, e_Read4, e_Read5, e_Read6, e_Read7);

signal PresentSdramTransferState, NextSdramTransferState    : t_SdramTransferStates;



type t_StatesCachePtrIn is (e_Start, e_WriteMsw, e_WriteLsw);
signal PresentStateCachePtrIn, NextStateCachePtrIn  : t_StatesCachePtrIn;

type t_StatesCachePtrOut is (e_Start, e_ReadMsw, e_Wait, e_ReadLsw);
signal PresentStateCachePtrOut, NextStateCachePtrOut  : t_StatesCachePtrOut;

begin

process(Clk)
begin
if Clk = '1' and Clk'event then
   ClkReset <= Reset;
end if;
end process;

--------------------------------------------------------------------------------------------------
--Cache pointer in/out statemachine
--------------------------------------------------------------------------------------------------
p_CachePtrIn : process(PresentStateCachePtrIn, FreePtrValidIn, PresentSdramTransferStateInit)
begin
NextStateCachePtrIn <= PresentStateCachePtrIn;
case PresentStateCachePtrIn is
when e_Start => 
   if FreePtrValidIn = '1' and PresentSdramTransferStateInit = e_EndInit  then
      NextStateCachePtrIn <= e_WriteMsw; 
   end if; 
when e_WriteMsw => 
   if FreePtrValidIn = '1' then 
      NextStateCachePtrIn <= e_WriteLsw; 
   end if;
when e_WriteLsw => NextStateCachePtrIn <= e_Start;
when others => NextStateCachePtrIn <= e_Start;
end case;
end process;
---
p_CachePtrOut : process(PresentStateCachePtrOut, GetFreePtr, PresentSdramTransferStateInit)
begin
NextStateCachePtrOut <= PresentStateCachePtrOut;
case PresentStateCachePtrOut is
when e_Start => NextStateCachePtrOut <= e_ReadMsw; 
when e_ReadMsw => 
   if GetFreePtr = '1' and PresentSdramTransferStateInit = e_EndInit  then
      NextStateCachePtrOut <= e_Wait; 
   end if;
when e_Wait => NextStateCachePtrOut <= e_ReadLsw;
when e_ReadLsw => 
   if GetFreePtr = '1' then 
      NextStateCachePtrOut <= e_Start; 
   end if;
when others => NextStateCachePtrOut <= e_Start;
end case;
end process;
---
------Add the statemachine to count the pointers

--------------------------------------------------------------------------------------------------
--SDRAM Initialization state machines
--------------------------------------------------------------------------------------------------
p_InitTransferRequest : process(PresentSdramRequestStateInit, 
                                SdramFreePtrPullReq, 
                                NumRequestsSent)
begin
NextSdramRequestStateInit <= PresentSdramRequestStateInit;
case PresentSdramRequestStateInit is
when e_StartInit => NextSdramRequestStateInit <= e_RequestInit; 
when e_RequestInit => NextSdramRequestStateInit <= e_WaitAckInit; 
when e_WaitAckInit => 
   if SdramFreePtrPullReq = '1' then 
      NextSdramRequestStateInit <= e_GotAckInit;
   end if;
when e_GotAckInit =>
      --if NumRequestsSent /= c_ActualFifoLengthInChunks then 
      if NumRequestsSent /= 100 then --just to make the simulation time short. In reality the statement above is to be used
         NextSdramRequestStateInit <= e_RequestInit;
      else
         NextSdramRequestStateInit <= e_EndInit;
      end if;
when others =>
   null;
end case;
end process;

----
p_InitTransfer : process(PresentSdramTransferStateInit, 
                         PresentSdramRequestStateInit,
                         SdramFreePtrGrant)
begin
NextSdramTransferStateInit <= PresentSdramTransferStateInit;
case PresentSdramTransferStateInit is
when e_StartInit     => NextSdramTransferStateInit <= e_WaitGrantInit;
when e_WaitGrantInit => 
   if SdramFreePtrGrant = '1' then 
      NextSdramTransferStateInit <= e_Write0Init; 
   end if;
when e_Write0Init    => NextSdramTransferStateInit <= e_Write1Init;
when e_Write1Init    => NextSdramTransferStateInit <= e_Write2Init;
when e_Write2Init    => NextSdramTransferStateInit <= e_Write3Init;
when e_Write3Init    => NextSdramTransferStateInit <= e_Write4Init;
when e_Write4Init    => NextSdramTransferStateInit <= e_Write5Init;
when e_Write5Init    => NextSdramTransferStateInit <= e_Write6Init;
when e_Write6Init    => NextSdramTransferStateInit <= e_Write7Init;
when e_Write7Init    => 
   if SdramFreePtrGrant = '1' then
      NextSdramTransferStateInit <= e_Write0Init;
   elsif PresentSdramRequestStateInit /= e_EndInit then 
      NextSdramTransferStateInit <= e_StartInit;
   else
      NextSdramTransferStateInit <= e_EndInit;
   end if;
when e_EndInit => NULL;
when others => NULL;
end case;
end process;


--------------------------------------------------------------------------------------------------
--Cache SDRAM transfer state machine
--------------------------------------------------------------------------------------------------

p_PtrTransferRequest : process(PresentSdramTransferState, 
                               PresentSdramTransferStateInit,
                               SdramFreePtrPullReq, 
                               CachePtrCount,
                               SdramFreePtrGrant, 
                               SdramFifoPtrCount
                               )
begin

NextSdramTransferState <= PresentSdramTransferState;

case PresentSdramTransferState is
---
when e_Start => 
   if PresentSdramTransferStateInit = e_EndInit  then
      NextSdramTransferState <= e_Check;
   end if;
when e_Check => 
   if CachePtrCount < c_Threshold1 and SdramFifoPtrCount /= 0 then
      NextSdramTransferState <= e_RequestRead;
   elsif CachePtrCount > c_Threshold2 then
      NextSdramTransferState <= e_RequestWrite;
   end if;

---
when e_RequestRead => NextSdramTransferState <= e_WaitAckRead;
when e_WaitAckRead => 
   if SdramFreePtrPullReq = '1' then
      NextSdramTransferState <= e_WaitGrantRead;
   end if;
when e_WaitGrantRead => 
   if SdramFreePtrGrant = '1' then
      NextSdramTransferState <= e_Read0;
   end if;
when e_Read0    => NextSdramTransferState<= e_Read1;
when e_Read1    => NextSdramTransferState<= e_Read2;
when e_Read2    => NextSdramTransferState<= e_Read3;
when e_Read3    => NextSdramTransferState<= e_Read4;
when e_Read4    => NextSdramTransferState<= e_Read5;
when e_Read5    => NextSdramTransferState<= e_Read6;
when e_Read6    => NextSdramTransferState<= e_Read7;
when e_Read7    => NextSdramTransferState<= e_Check;

---
when e_RequestWrite => NextSdramTransferState <= e_WaitAckWrite;
when e_WaitAckWrite =>
   if SdramFreePtrPullReq = '1' then
      NextSdramTransferState <= e_WaitGrantWrite;
   end if;
when e_WaitGrantWrite =>
   if SdramFreePtrGrant = '1' then
      NextSdramTransferState <= e_Write0;
   end if;
when e_Write0    => NextSdramTransferState<= e_Write1;
when e_Write1    => NextSdramTransferState<= e_Write2;
when e_Write2    => NextSdramTransferState<= e_Write3;
when e_Write3    => NextSdramTransferState<= e_Write4;
when e_Write4    => NextSdramTransferState<= e_Write5;
when e_Write5    => NextSdramTransferState<= e_Write6;
when e_Write6    => NextSdramTransferState<= e_Write7;
when e_Write7    => NextSdramTransferState<= e_Check;

when others =>
   null;
end case;
end process;

--------------------------------------------------------------------------------------------------
--Clocking the states
--------------------------------------------------------------------------------------------------
process(Clk)
begin
if Clk = '1' and Clk'event then 
   if ClkReset = '0' then
      PresentStateCachePtrIn  <= e_Start; 
      PresentStateCachePtrOut  <= e_ReadMsw; 
      PresentSdramRequestStateInit  <= e_StartInit; 
      PresentSdramTransferStateInit  <= e_StartInit; 
      PresentSdramTransferState  <= e_Start; 
   else
      PresentStateCachePtrIn  <= NextStateCachePtrIn;
      PresentStateCachePtrOut  <= NextStateCachePtrOut; 
      PresentSdramRequestStateInit  <= NextSdramRequestStateInit; 
      PresentSdramTransferStateInit  <= NextSdramTransferStateInit; 
      PresentSdramTransferState  <= NextSdramTransferState; 
   end if;
end if;
end process;


myfreeptrs : freeptrs
        port map (
            clka => Clk,
            addra => addra,
            douta => douta,
            clkb => Clk,
            addrb => addrb,
            dinb => dinb,
            doutb => doutb,
            web => web);

process(Clk)

variable v_CachePtrCount : integer range 0 to 512;
begin

if Clk = '1' and Clk'event then
  if ClkReset = '0' then

     Initialization <= '1';
     --Outputs : To ChunkWriter
     FreePtrValidOut <= '0';
     FreePtrOut <= (others => '0');
     BufferFull <= '0';
     BufferFull1 <= '0';

     --Outputs : To PacketReader
     WriteError <= '0';

     --Outputs : To SDRAM controller
     SdramFreePtrs       <= (others => 'Z');
     SdramFreePtrFifoAddr<= (others => '0');
     SdramFreePtrOp      <= '0';
     SdramFreePtrOpValid <= '0';

     --Internal signals : SDRAM FIFO
     SdramFifoReadPointer <= c_SdramFifoStartMarker + conv_std_logic_vector(c_PtrOnchip/2,c_AddressWidth); 
     SdramFifoWritePointer<= c_SdramFifoStartMarker; 
     InitializeFifoCntr   <= (others => '0');
     NumRequestsSent      <= 0;
     SdramFifoPtrCount    <= 0;

     --Internal signals : Cache  FIFO
     addrb <= (others => '1');
     dinb  <= (others => '0');
     web   <= '0';
     addra <= (others => '0');
     CachePtrCount   <= 0;
     v_CachePtrCount := 0;
     TmpCount  <= c_Queues + 1;
     InitializeBram <= '1';

     ClkFreePtrIn <= (others => '0');
  else


     SdramFreePtrs       <= (others => 'Z');
     v_CachePtrCount := CachePtrCount;
----------------------------------------------------------------------------------------------
--Initialization Section
----------------------------------------------------------------------------------------------
    --Initialize the BRAM Fifo
    web <= '0';
    if InitializeBram = '1' then
       if web = '1' and addrb = "11111111" then
          InitializeBram <= '0';
          web <= '0';
       else
          v_CachePtrCount := v_CachePtrCount + 2;
          web <= '1';
          addrb <= addrb + '1';
          dinb(63 downto 32) <= conv_std_logic_vector(TmpCount,32);
          dinb(31 downto 0) <= conv_std_logic_vector(TmpCount + 1,32);
          TmpCount <= TmpCount + 2;
       end if;
    end if;

     --Initialization of the SDRAM FIFO Request statemachine
     if PresentSdramRequestStateInit = e_StartInit then
        NULL;
     elsif PresentSdramRequestStateInit = e_RequestInit then
        NumRequestsSent       <= NumRequestsSent + 1;
        SdramFreePtrFifoAddr  <= SdramFifoWritePointer;
        SdramFreePtrOp        <= '1';
        SdramFreePtrOpValid   <= '1';
     elsif PresentSdramRequestStateInit = e_WaitAckInit then
        if SdramFreePtrPullReq = '1' then
           SdramFreePtrOpValid   <= '0';
        end if;
     elsif PresentSdramRequestStateInit = e_GotAckInit then
        SdramFifoWritePointer <= SdramFifoWritePointer + "1000";
     elsif PresentSdramRequestStateInit = e_EndInit then
        NULL;
     end if;

     --Initialization of the SDRAM FIFO Transfer statemachine
     if PresentSdramTransferStateInit = e_StartInit then
        NULL;
     elsif PresentSdramTransferStateInit = e_WaitGrantInit then
        NULL;
     elsif PresentSdramTransferStateInit = e_Write0Init then
        SdramFreePtrs  <= InitializeFifoCntr & (InitializeFifoCntr + '1'); 
        InitializeFifoCntr <= InitializeFifoCntr + "10";
     elsif PresentSdramTransferStateInit = e_Write1Init then
        SdramFreePtrs  <= InitializeFifoCntr & (InitializeFifoCntr + '1'); 
        InitializeFifoCntr <= InitializeFifoCntr + "10";
     elsif PresentSdramTransferStateInit = e_Write2Init then
        SdramFreePtrs  <= InitializeFifoCntr & (InitializeFifoCntr + '1'); 
        InitializeFifoCntr <= InitializeFifoCntr + "10";
     elsif PresentSdramTransferStateInit = e_Write3Init then
        SdramFreePtrs  <= InitializeFifoCntr & (InitializeFifoCntr + '1'); 
        InitializeFifoCntr <= InitializeFifoCntr + "10";
     elsif PresentSdramTransferStateInit = e_Write4Init then
        SdramFreePtrs  <= InitializeFifoCntr & (InitializeFifoCntr + '1'); 
        InitializeFifoCntr <= InitializeFifoCntr + "10";
     elsif PresentSdramTransferStateInit = e_Write5Init then
        SdramFreePtrs  <= InitializeFifoCntr & (InitializeFifoCntr + '1'); 
        InitializeFifoCntr <= InitializeFifoCntr + "10";
     elsif PresentSdramTransferStateInit = e_Write6Init then
        SdramFreePtrs  <= InitializeFifoCntr & (InitializeFifoCntr + '1'); 
        InitializeFifoCntr <= InitializeFifoCntr + "10";
     elsif PresentSdramTransferStateInit = e_Write7Init then
        SdramFreePtrs  <= InitializeFifoCntr & (InitializeFifoCntr + '1'); 
        InitializeFifoCntr <= InitializeFifoCntr + "10";
        SdramFifoPtrCount    <= SdramFifoPtrCount + 16;
     elsif PresentSdramTransferStateInit = e_EndInit then
        Initialization <= '0';
        NULL;
     end if;

----------------------------------------------------------------------------------------------
--Pointer cache in BRAM
----------------------------------------------------------------------------------------------
     if FreePtrValidIn = '1' then
        ClkFreePtrIn <= FreePtrIn;
     end if;
     if PresentStateCachePtrIn = e_Start then
        NULL;
     elsif PresentStateCachePtrIn = e_WriteMsw then
        dinb(63 downto 32) <= ClkFreePtrIn;
        --web <= '1';
     elsif PresentStateCachePtrIn = e_WriteLsw then
        dinb(31 downto 0) <= ClkFreePtrIn;
        web <= '1';
        addrb <= addrb + '1';
        v_CachePtrCount := v_CachePtrCount +2;
     end if;

        FreePtrValidOut <= '0';
     --if PresentStateCachePtrOut = e_Start then
     --   NULL;
     if PresentStateCachePtrOut = e_ReadMsw then
        if GetFreePtr = '1' and PresentSdramTransferStateInit = e_EndInit  then
           FreePtrValidOut <= '1';
           FreePtrOut <= douta(63 downto 32);
        end if;
     elsif PresentStateCachePtrOut = e_ReadLsw then
        if GetFreePtr = '1' then
           FreePtrValidOut <= '1';
           FreePtrOut <= douta(31 downto 0);
           addra <= addra + '1';
           v_CachePtrCount := v_CachePtrCount -2;
        end if;
     end if;

---------------------------------------------------------------------------------------
----Cache - Sdram interaction
---------------------------------------------------------------------------------------
     if PresentSdramTransferState = e_Start then
        NULL;
     elsif PresentSdramTransferState = e_Check then
        NULL;
     elsif PresentSdramTransferState = e_RequestRead then
        SdramFreePtrFifoAddr  <= SdramFifoReadPointer;
        SdramFreePtrOp        <= '0';
        SdramFreePtrOpValid   <= '1';
     elsif PresentSdramTransferState = e_WaitAckRead then
        if SdramFreePtrPullReq = '1' then
           SdramFreePtrOpValid   <= '0';
           SdramFifoReadPointer <= SdramFifoReadPointer + 8;
        end if;
     elsif PresentSdramTransferState = e_WaitGrantRead then
        NULL;
     elsif PresentSdramTransferState = e_Read0 then
        dinb <= SdramFreePtrs;
        web  <= '1';
        addrb <= addrb + '1';
        v_CachePtrCount := v_CachePtrCount +2;
        SdramFifoPtrCount    <= SdramFifoPtrCount - 2;
     elsif PresentSdramTransferState = e_Read1 then
        dinb <= SdramFreePtrs;
        web  <= '1';
        addrb <= addrb + '1';
        v_CachePtrCount := v_CachePtrCount +2;
        SdramFifoPtrCount    <= SdramFifoPtrCount - 2;
     elsif PresentSdramTransferState = e_Read2 then
        dinb <= SdramFreePtrs;
        web  <= '1';
        addrb <= addrb + '1';
        v_CachePtrCount := v_CachePtrCount +2;
        SdramFifoPtrCount    <= SdramFifoPtrCount - 2;
     elsif PresentSdramTransferState = e_Read3 then
        dinb <= SdramFreePtrs;
        web  <= '1';
        addrb <= addrb + '1';
        v_CachePtrCount := v_CachePtrCount +2;
        SdramFifoPtrCount    <= SdramFifoPtrCount - 2;
     elsif PresentSdramTransferState = e_Read4 then
        dinb <= SdramFreePtrs;
        web  <= '1';
        addrb <= addrb + '1';
        v_CachePtrCount := v_CachePtrCount +2;
        SdramFifoPtrCount    <= SdramFifoPtrCount - 2;
     elsif PresentSdramTransferState = e_Read5 then
        dinb <= SdramFreePtrs;
        web  <= '1';
        addrb <= addrb + '1';
        v_CachePtrCount := v_CachePtrCount +2;
        SdramFifoPtrCount    <= SdramFifoPtrCount - 2;
     elsif PresentSdramTransferState = e_Read6 then
        dinb <= SdramFreePtrs;
        web  <= '1';
        addrb <= addrb + '1';
        v_CachePtrCount := v_CachePtrCount +2;
        SdramFifoPtrCount    <= SdramFifoPtrCount - 2;
     elsif PresentSdramTransferState = e_Read7 then
        dinb <= SdramFreePtrs;
        web  <= '1';
        addrb <= addrb + '1';
        v_CachePtrCount := v_CachePtrCount +2;
        SdramFifoPtrCount    <= SdramFifoPtrCount - 2;
     elsif PresentSdramTransferState = e_RequestWrite then
        SdramFreePtrFifoAddr  <= SdramFifoWritePointer;
        SdramFreePtrOp        <= '1';
        SdramFreePtrOpValid   <= '1';
     elsif PresentSdramTransferState = e_WaitAckWrite then
        if SdramFreePtrPullReq = '1' then
           SdramFreePtrOpValid   <= '0';
           SdramFifoWritePointer <= SdramFifoWritePointer + 8;
        end if;
     elsif PresentSdramTransferState = e_WaitGrantWrite then
        if SdramFreePtrGrant = '1' then
           addrb <= addrb - '1';
        end if;
     elsif PresentSdramTransferState = e_Write0 then
        SdramFreePtrs <= doutb;
        addrb <= addrb - '1';
        v_CachePtrCount := v_CachePtrCount -2;
        SdramFifoPtrCount    <= SdramFifoPtrCount + 2;
     elsif PresentSdramTransferState = e_Write1 then
        SdramFreePtrs <= doutb;
        addrb <= addrb - '1';
        v_CachePtrCount := v_CachePtrCount -2;
        SdramFifoPtrCount    <= SdramFifoPtrCount + 2;
     elsif PresentSdramTransferState = e_Write2 then
        SdramFreePtrs <= doutb;
        addrb <= addrb - '1';
        v_CachePtrCount := v_CachePtrCount -2;
        SdramFifoPtrCount    <= SdramFifoPtrCount + 2;
     elsif PresentSdramTransferState = e_Write3 then
        SdramFreePtrs <= doutb;
        addrb <= addrb - '1';
        v_CachePtrCount := v_CachePtrCount -2;
        SdramFifoPtrCount    <= SdramFifoPtrCount + 2;
     elsif PresentSdramTransferState = e_Write4 then
        SdramFreePtrs <= doutb;
        addrb <= addrb - '1';
        v_CachePtrCount := v_CachePtrCount -2;
        SdramFifoPtrCount    <= SdramFifoPtrCount + 2;
     elsif PresentSdramTransferState = e_Write5 then
        SdramFreePtrs <= doutb;
        addrb <= addrb - '1';
        v_CachePtrCount := v_CachePtrCount -2;
        SdramFifoPtrCount    <= SdramFifoPtrCount + 2;
     elsif PresentSdramTransferState = e_Write6 then
        SdramFreePtrs <= doutb;
        addrb <= addrb - '1';
        v_CachePtrCount := v_CachePtrCount -2;
        SdramFifoPtrCount    <= SdramFifoPtrCount + 2;
     elsif PresentSdramTransferState = e_Write7 then
        SdramFreePtrs <= doutb;
        v_CachePtrCount := v_CachePtrCount -2;
        SdramFifoPtrCount    <= SdramFifoPtrCount + 2;
     end if;

     CachePtrCount <= v_CachePtrCount;

     --Indicate when the buffer is full
     if CachePtrCount = 0 and SdramFifoPtrCount = 0 then
        BufferFull1 <= '1';
     else
        BufferFull1 <= '0';
     end if;
     BufferFull <= BufferFull1;
  end if; --if reset = 0
end if; --if Clk = 1

end process;
end behave;
