library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library work;
use work.PkgSdramController.all;
use work.PkgPsm.all;

entity InputAdapter is
port (
      --Interface with the IP wrapper
      PktDataIn  : in std_logic_vector(31 downto 0);
      PktDataInValid  : in std_logic;
      SoPktIn  : in std_logic;
      EoPktIn  : in std_logic;

      --Interface with the Queue Contex
      Tail : in std_logic_vector(31 downto 0);
      TailValid : in std_logic;
      NextTail : out std_logic_vector(31 downto 0);
      NextTailValid : out std_logic;

      --Interface with the PSM
      SokPsm : out std_logic;
      SopPsm : out std_logic;
      EopPsm : out std_logic;
      PktDataPsm  : out std_logic_vector(63 downto 0);

      TailPsm : out std_logic_vector(31 downto 0);
      TailValidPsm : out std_logic;

      NextTailPsm : in std_logic_vector(31 downto 0);
      NextTailValidPsm : in std_logic;

      Clk : in std_logic;
      Reset : in std_logic

);
end InputAdapter;

architecture behave of InputAdapter is

component my_data_fifo 
        port (
        clk: IN std_logic;
        sinit: IN std_logic;
        din: IN std_logic_VECTOR(63 downto 0);
        wr_en: IN std_logic;
        rd_en: IN std_logic;
        dout: OUT std_logic_VECTOR(63 downto 0);
        full: OUT std_logic;
        data_count : OUT std_logic_vector(9 downto 0);
        empty: OUT std_logic);
END component;
-- Synplicity black box declaration
attribute syn_black_box : boolean;
attribute syn_black_box of my_data_fifo: component is true;



component my_request_fifo 
        port (
        clk: IN std_logic;
        sinit: IN std_logic;
        din: IN std_logic_VECTOR(31 downto 0);
        wr_en: IN std_logic;
        rd_en: IN std_logic;
        dout: OUT std_logic_VECTOR(31 downto 0);
        full: OUT std_logic;
        empty: OUT std_logic);
END component;
attribute syn_black_box of my_request_fifo: component is true;

--Signals for Tail FIFO
signal NotReset : std_logic;
signal RequestQFull : std_logic;
signal RequestQEmpty : std_logic;
signal ReadTail : std_logic;

--signals for length FIFO
signal PktLength : std_logic_vector(15 downto 0);

--SIgnals for ChunkFIFO
signal TmpPktDataIn : std_logic_vector(c_DataWidth-1 downto 0);
signal TmpPktDataInValid : std_logic;
signal ReadPktData : std_logic;
signal PktQFull : std_logic;
signal PktQEmpty : std_logic;
signal BytesWritten : integer range 0 to 124;
signal FillMsb : std_logic;


signal BytesToRead : integer range 0 to 2000; 

type t_State is (e_Start, e_Pull, e_LatchLength, e_OperateLength, e_Decide, 
e_r0,e_r1,e_r2,e_r3,e_r4,e_r5,e_r6,e_r7,e_r8,e_r9,e_r10,e_r11,e_r12,e_r13,e_r14,e_r15);
signal PresentState, NextState : t_State;

signal ChunkFifoDataCount : std_logic_vector(9 downto 0);
signal WordsToRead : integer range 0 to 2000;
signal NewPacket : std_logic;
signal TmpPktDataPsm : std_logic_vector(c_DataWidth-1 downto 0);
begin

NextTail <= NextTailPsm;
NextTailValid <= NextTailValidPsm;
NotReset <= not Reset;
PktDataPsm <= TmpPktDataPsm;

MyRequestFifo: my_request_fifo
        port map(
        clk => Clk, 
        sinit => NotReset, 
        din => Tail, 
        wr_en => TailValid, 
        rd_en => ReadTail, 
        dout => TailPsm, 
        full => RequestQFull, 
        empty => RequestQEmpty); 

MyChunkFifo :  my_data_fifo 
        port map(
        clk =>  Clk, 
        sinit => NotReset,  
        din =>  TmpPktDataIn, 
        wr_en => TmpPktDataInValid, 
        rd_en => ReadPktData, 
        dout => TmpPktDataPsm, 
        full => PktQFull, 
        empty => PktQEmpty,
        data_count => ChunkFifoDataCount); 

--This process writes the formatted data in the temporary chunk FIFO
process(Clk)
begin
if Clk = '1' and Clk'event then
   if Reset = '0' then
      TmpPktDataIn <= (others => '0');
      TmpPktDataInValid <= '0';
      BytesWritten <= 124;
      FillMsb <= '1';
   else
      TmpPktDataInValid <= '0';
      if PktDataInValid = '1' then
         if SoPktIn = '1' or BytesWritten = 124 then
            TmpPktDataIn(63 downto 32) <= (others => '0');
            TmpPktDataIn(31 downto 0) <= PktDataIn;
            TmpPktDataInValid <= '1';
            BytesWritten <= 4;
            FillMsb <= '1';
         else
            BytesWritten <= BytesWritten + 4;
            if FillMsb = '1' then
               TmpPktDataIn(63 downto 32) <= PktDataIn;
               FillMsb <= '0';
            else
               TmpPktDataIn(31 downto 0) <= PktDataIn;
               FillMsb <= '1';
               TmpPktDataInValid <= '1';
            end if;

            if EoPktIn = '1' then
               TmpPktDataInValid <= '1';
               BytesWritten <= 0;
               FillMsb <= '0';
            end if;
         end if;
      end if;

   end if; --if Reset
end if; --if Clk
end process;


---The following process pulls the data out of the data fifo at appropriate time

process(Clk)
variable v_TmpPktLength : std_logic_vector(15 downto 0);
begin
if Clk = '1' and Clk'event then
   if Reset = '0' then
      PresentState <= e_Start;

      BytesToRead <= 0;
      WordsToRead <= 0;

      ReadPktData <= '0';
      ReadTail <= '0'; 

      SokPsm <= '0'; 
      SopPsm <= '0'; 
      EopPsm <= '0'; 
      TailValidPsm <= '0';
      NewPacket <= '1';
   else
      ReadTail <= '0'; 
      ReadPktData <= '0';

      SokPsm <= '0'; 
      SopPsm <= '0'; 
      EopPsm <= '0'; 
      TailValidPsm <= '0';

      if PresentState = e_Start then
         if NewPacket = '1' then
            if PktQEmpty = '0' and RequestQEmpty = '0' then
               PresentState <= e_Pull;
               ReadPktData <= '1';
            end if; 
         else
            if PktQEmpty = '0' then
               PresentState <= e_Pull;
               ReadPktData <= '1';
            end if; 
         end if;
      elsif PresentState = e_Pull then
         if NewPacket = '1' then
            PresentState <= e_LatchLength;
            ReadTail <= '1'; 
         else
            PresentState <= e_OperateLength;
         end if;
      elsif PresentState = e_LatchLength then
         BytesToRead <= conv_integer(TmpPktDataPsm(15 downto 0)) + 8 ; --4 taken out for the first word and 8 added 
                                                                         --for the last two words of the frame it sucks 
                                                                         --but it is okey
         PresentState <= e_OperateLength;
      elsif PresentState = e_OperateLength then
         PresentState <= e_Decide;
         if BytesToRead > 124 then
            BytesToRead <= BytesToRead - 124;
            WordsToRead <= 15;
         else
            BytesToRead <= 0;
            if BytesToRead > 4 then
               v_TmpPktLength := conv_std_logic_vector((BytesToRead-4),16);
               WordsToRead <= conv_integer(v_TmpPktLength(15 downto 3) +    
                                           (v_TmpPktLength(2) OR v_TmpPktLength(1) OR v_TmpPktLength(0)));
            else
               WordsToRead <= 0;
            end if; 
         end if; 
      elsif PresentState = e_Decide then
         if (conv_integer(ChunkFifoDataCount) >= 14) or (conv_integer(ChunkFifoDataCount)>=WordsToRead) then
            PresentState <= e_r0;
            SokPsm <= '1';
            if NewPacket = '1' then
               SopPsm <= '1';
               TailValidPsm <= '1';
            end if;
            if BytesToRead = 0 then
               EopPsm <= '1';
               NewPacket <= '1';
            else
               NewPacket <= '0';
            end if;
            if WordsToRead > 0 then
               ReadPktData <= '1';
               WordsToRead <= WordsToRead - 1;
            end if;
         end if;
      elsif PresentState = e_r0 then
         PresentState <= e_r1;
         if WordsToRead > 0 then
            ReadPktData <= '1';
            WordsToRead <= WordsToRead - 1;
         end if;
      elsif PresentState = e_r1 then
         PresentState <= e_r2;
         if WordsToRead > 0 then
            ReadPktData <= '1';
            WordsToRead <= WordsToRead - 1;
         end if;
      elsif PresentState = e_r2 then
         PresentState <= e_r3;
         if WordsToRead > 0 then
            ReadPktData <= '1';
            WordsToRead <= WordsToRead - 1;
         end if;
      elsif PresentState = e_r3 then
         PresentState <= e_r4;
         if WordsToRead > 0 then
            ReadPktData <= '1';
            WordsToRead <= WordsToRead - 1;
         end if;
      elsif PresentState = e_r4 then
         PresentState <= e_r5;
         if WordsToRead > 0 then
            ReadPktData <= '1';
            WordsToRead <= WordsToRead - 1;
         end if;
      elsif PresentState = e_r5 then
         PresentState <= e_r6;
         if WordsToRead > 0 then
            ReadPktData <= '1';
            WordsToRead <= WordsToRead - 1;
         end if;
      elsif PresentState = e_r6 then
         PresentState <= e_r7;
         if WordsToRead > 0 then
            ReadPktData <= '1';
            WordsToRead <= WordsToRead - 1;
         end if;
      elsif PresentState = e_r7 then
         PresentState <= e_r8;
         if WordsToRead > 0 then
            ReadPktData <= '1';
            WordsToRead <= WordsToRead - 1;
         end if;
      elsif PresentState = e_r8 then
         PresentState <= e_r9;
         if WordsToRead > 0 then
            ReadPktData <= '1';
            WordsToRead <= WordsToRead - 1;
         end if;
      elsif PresentState = e_r9 then
         PresentState <= e_r10;
         if WordsToRead > 0 then
            ReadPktData <= '1';
            WordsToRead <= WordsToRead - 1;
         end if;
      elsif PresentState = e_r10 then
         PresentState <= e_r11;
         if WordsToRead > 0 then
            ReadPktData <= '1';
            WordsToRead <= WordsToRead - 1;
         end if;
      elsif PresentState = e_r11 then
         PresentState <= e_r12;
         if WordsToRead > 0 then
            ReadPktData <= '1';
            WordsToRead <= WordsToRead - 1;
         end if;
      elsif PresentState = e_r12 then
         PresentState <= e_r13;
         if WordsToRead > 0 then
            ReadPktData <= '1';
            WordsToRead <= WordsToRead - 1;
         end if;
      elsif PresentState = e_r13 then
         PresentState <= e_r14;
         if WordsToRead > 0 then
            ReadPktData <= '1';
            WordsToRead <= WordsToRead - 1;
         end if;
      elsif PresentState = e_r14 then
         PresentState <= e_Start;
         if WordsToRead > 0 then
            ReadPktData <= '1';
            WordsToRead <= WordsToRead - 1;
         end if;
      end if;


   end if;
end if;
end process;

end behave;
