--for the packet reader. The read requests include the packet read
--requests, the individual chunk read requetsts, and the discard
--packet requests. The write requests involve the one word, copy
--count update.

--The request creater takes the input from the discard queue,
--packet read queue and the data that is read from the SDRAM
--and produces the appropriate requests for the packet reader.

--The discard queue is given least priority. First a packet pointer
--is pulled out of the ReadQueue and a request for the first chunk 
--of the packet is created. If the packet needs the copy count
--update then immediately a one word write requests is created.
--When the first chunk is read out and its next pointer indicates
--that there are more chunks in the packet then a read requests
--for the next chunk is created. The request includes the
--address, burst length and the type of request. 

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

library work;
use work.PkgSdramController.all;
use work.PkgPsm.all;


entity PacketReader is
port (

       --Interface with OSAR
       ValidPtrInOsar          : in std_logic;
       PacketPtrInOsar         : in std_logic_vector(c_ChunkPointerWidth-1 downto 0);

       SokOutOsar            : out std_logic;
       SopOutOsar            : out std_logic;
       EopOutOsar            : out std_logic;
       DataOutOsar           : out std_logic_vector(c_DataWidth-1 downto 0);
       NextPacketPtrOutOsar   :  out std_logic_vector(c_ChunkPointerWidth-1 downto 0); 
       NextPacketPtrValidOutOsar : out std_logic; 

       --Interface with the Sequencer
       RequestValid      : out std_logic;
       Address           : out std_logic_vector(c_AddressWidth-1 downto 0);
       BurstLength       : out std_logic_vector(7 downto 0);
       GiveRequest       : in std_logic;

       --Interface with the FreePointerManager
       Initialization     : in std_logic;
       FreePtr           : out std_logic_vector(c_ChunkPointerWidth-1 downto 0);
       FreePtrValid      : out std_logic;
       WriteError        : in std_logic;

       --Interface with the Sdram Data bus
       SdramGrant        : in std_logic;
       SdramData         : in std_logic_vector(c_DataWidth-1 downto 0);

       Clk               : in std_logic;
       Reset             : in std_logic);
end PacketReader;


architecture behave of PacketReader is

signal ClkReset : std_logic;

type t_RequestCreaterState is ( e_Start, e_PullReqFromRdQ, e_PullReqFromDiscQ, e_FirstChunkReq, e_Wait, 
                                e_NextChunkInspect, e_NextChunkReq);
signal RequestCreaterState : t_RequestCreaterState;
signal NextRequestCreaterState : t_RequestCreaterState;

signal ClkSdramData         : std_logic_vector(c_DataWidth-1 downto 0);
alias PacketEndMarker : std_logic is SdramData(c_DataWidth-1);
alias NextPointer : std_logic_vector(c_ChunkPointerWidth-1 downto 0) is ClkSdramData(c_DataWidth-1 downto c_DataWidth/2);

signal TempPacketPtr         : std_logic_vector(c_ChunkPointerWidth-1 downto 0);

       --Interface with the RequetsQueue
signal  GetReadRequest    : std_logic;
signal  ReadQueueEmpty    : std_logic;
signal  ReadPacketPtr     : std_logic_vector(c_ChunkPointerWidth-1 downto 0);

signal FirstChunk         : std_logic;
signal  PtrToDeallocate  : std_logic_vector(c_ChunkPointerWidth-1 downto 0);
type t_PacketType is (e_NormalPacket);
signal TempPacketType : t_PacketType;
signal BiggerPacket : std_logic;

begin

ReadQueueFsm : process(Clk)
begin
if Clk = '1' and Clk'event then
   if ClkReset = '0' then
      ReadQueueEmpty <= '1';
      ReadPacketPtr <= (others =>'0');
   else
      if ValidPtrInOsar = '1' then
         ReadPacketPtr <= PacketPtrInOsar;
         ReadQueueEmpty <= '0';
      elsif GetReadRequest = '1' then
         ReadQueueEmpty <= '1';
         ReadPacketPtr <= (others =>'0');
      end if;
   end if;
end if;
end process;


----------------------------------------------------------------------------------
--The FMS for creating requests
----------------------------------------------------------------------------------
process(RequestCreaterState, 
        ReadQueueEmpty, 
        SdramGrant, 
        PacketEndMarker, 
        Initialization)

begin

NextRequestCreaterState <= RequestCreaterState;

case RequestCreaterState is
when e_Start => if Initialization = '0' then 
                   if ReadQueueEmpty = '0' then
                      NextRequestCreaterState <= e_PullReqFromRdQ;
                   end if; 
                end if;
when e_PullReqFromRdQ => NextRequestCreaterState <= e_FirstChunkReq;
                            
when e_FirstChunkReq => NextRequestCreaterState <= e_Wait;
                       
when e_Wait => if SdramGrant = '1' then
                  NextRequestCreaterState <= e_NextChunkInspect;
               end if;
when e_NextChunkInspect => 
                  if PacketEndMarker = '1' then
                     NextRequestCreaterState <= e_Start;
                  else 
                     NextRequestCreaterState <= e_NextChunkReq;
                  end if;
when e_NextChunkReq =>   NextRequestCreaterState <= e_Wait;
                 
when others => NextRequestCreaterState <= e_Start;
end case;

end process;

process(Clk)
begin
if Clk = '1' and Clk'event then
   if ClkReset = '0' then
      RequestCreaterState <= e_Start;
   else
      RequestCreaterState <= NextRequestCreaterState;
   end if;
end if;
end process;
----------------------------------------------------------------------------------
--Actions taken in the create request statemachine
----------------------------------------------------------------------------------


process(Clk)

begin
if Clk = '1' and Clk'event then
   if ClkReset = '0' then

      Address       <= (others => '0');
      BurstLength   <= (others => '0');

      TempPacketPtr         <= (others => '0'); 
      ClkSdramData          <= (others => '0');
      GetReadRequest        <= '0';
      SokOutOsar                <= '0';
      SopOutOsar                <= '0'; 
      EopOutOsar                <= '0'; 
      DataOutOsar               <= (others => '0'); 
      FreePtr               <= (others => '0');
      FreePtrValid          <= '0'; 
      FirstChunk            <= '0';
      PtrToDeallocate          <= (others => '0');
      TempPacketType        <= e_NormalPacket;
      BiggerPacket          <= '0';
      NextPacketPtrOutOsar <= (others => '0');
      NextPacketPtrValidOutOsar <= '0';
      RequestValid        <= '0'; 
   else
      BurstLength <= "00001111";
      ClkSdramData          <= SdramData;
      GetReadRequest <= '0';

----------------------------------------------------------------------------------
--Creating a request
----------------------------------------------------------------------------------



      if RequestCreaterState = e_PullReqFromRdQ then
            TempPacketPtr    <= ReadPacketPtr;     
            TempPacketType   <= e_NormalPacket;
            GetReadRequest   <= '1';
     ---------------------------------------------------------------------------------------
      elsif RequestCreaterState = e_FirstChunkReq then
            Address      <= TempPacketPtr(c_Addresswidth-5 downto 0) & "0000" ;
            RequestValid <= '1'; 
            FirstChunk <= '1';
     ---------------------------------------------------------------------------------------
      elsif RequestCreaterState = e_NextChunkReq then
            Address      <= NextPointer(c_Addresswidth-5 downto 0) & "0000" ;
            RequestValid <= '1'; 
     end if; --if presentstate 


----------------------------------------------------------------------------------
--Pulling a request
----------------------------------------------------------------------------------
      if GiveRequest = '1' then
         Address      <= (others => '0');
         RequestValid <= '0'; 
      end if;

----------------------------------------------------------------------------------
--Outputing data
----------------------------------------------------------------------------------

     SokOutOsar  <= '0';
     EopOutOsar  <= '0';
     SopOutOsar  <= '0';
     DataOutOsar <= SdramData;
     NextPacketPtrOutOsar <= (others => '0');
     NextPacketPtrValidOutOsar <= '0';
     if TempPacketType = e_NormalPacket then
        if RequestCreaterState = e_NextChunkInspect then
           SokOutOsar <= '1';
           if PacketEndMarker = '1' then
              EopOutOsar <= '1';
              NextPacketPtrOutOsar <= '0'&SdramData(c_DataWidth-2 downto c_DataWidth/2);
              NextPacketPtrValidOutOsar <= '1';
           end if;
           if FirstChunk = '1' then
              FirstChunk <= '0';
              SopOutOsar <= '1';
           end if;
        end if;
     end if;
 
----------------------------------------------------------------------------------
--Releasing the pointer
----------------------------------------------------------------------------------
    FreePtr <= (others => '0');
    FreePtrValid <= '0';
    if RequestCreaterState = e_FirstChunkReq then
       PtrToDeallocate <= TempPacketPtr;
    elsif RequestCreaterState = e_NextChunkInspect then
       FreePtr <= PtrToDeallocate;
       FreePtrValid <= '1';
    elsif RequestCreaterState = e_NextChunkReq then
       PtrToDeallocate <= NextPointer;
    end if; 



   end if;--if reset = 0
end if;
end process;      


process(Clk)
begin
if Clk = '1' and Clk'event then
   ClkReset <= Reset;
end if;
end process;

end behave;

