
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library work;
use work.PkgSdramController.all;
use work.PkgPsm.all;

entity Psm is
port (

       Initialization: out std_logic;
       --From ISAR
       SopInIsar         : in std_logic;
       SokInIsar         : in std_logic;
       EopInIsar         : in std_logic;
       DataInIsar        : in std_logic_vector(c_DataWidth-1 downto 0);
       PacketPtrInIsar     : in std_logic_vector(c_ChunkPointerWidth-1 downto 0);
       NextPacketPtrOutIsar       : out std_logic_vector(c_ChunkPointerWidth-1 downto 0);
       NextPacketPtrValidOutIsar  : out std_logic;
       BufferFullOutIsar  : out std_logic;


       --Interface with OSAR
       ValidPtrInOsar          : in std_logic;
       PacketPtrInOsar         : in std_logic_vector(c_ChunkPointerWidth-1 downto 0);

       SokOutOsar            : out std_logic;
       SopOutOsar            : out std_logic;
       EopOutOsar            : out std_logic;
       DataOutOsar           : out std_logic_vector(c_DataWidth-1 downto 0);
       NextPacketPtrOutOsar   :  out std_logic_vector(c_ChunkPointerWidth-1 downto 0);
       NextPacketPtrValidOutOsar : out std_logic;


       --To the SDRAM
       SdramAddr               : out   std_logic_vector(11 downto 0);
       SdramData               : inout std_logic_vector(c_DataWidth -1 downto 0);
       SdramRas                : out   std_logic;
       SdramCas                : out   std_logic;
       SdramCe                 : out   std_logic_vector(1 downto 0);
       SdramBa                 : out   std_logic_vector(1 downto 0);
       SdramS                  : out   std_logic_vector(1 downto 0);
       SdramWe                 : out   std_logic;

       Clk                     : in std_logic;
       Reset                   : in std_logic);
end Psm;


architecture struct of Psm is

component ChunkWriter 
port (
       --From the ISAR
       SopInIsar         : in std_logic;
       SokInIsar         : in std_logic;
       EopInIsar         : in std_logic;
       PacketPtrInIsar     : in std_logic_vector(c_ChunkPointerWidth-1 downto 0);
       DataInIsar        : in std_logic_vector(c_DataWidth-1 downto 0);
       NextPacketPtrOutIsar       : out std_logic_vector(c_ChunkPointerWidth-1 downto 0);
       NextPacketPtrValidOutIsar  : out std_logic;
       BufferFullOutIsar  : out std_logic;

       --To free pointer manager
       Initialization : in std_logic;
       GetFreePtr    : out std_logic;
       FreePtrValid  : in std_logic;
       BufferFull    : in std_logic;
       FreePtr       : in std_logic_vector(c_ChunkPointerWidth-1 downto 0);

       --To SDRAM controller
       AddressOut    : out std_logic_vector(c_AddressWidth-1 downto 0);
       BurstLengthOut : out std_logic_vector(7 downto 0);
       DataOut       : out std_logic_vector(c_DataWidth-1 downto 0);
       ChunkAvailable: out std_logic;
       GetData       : in std_logic;
       PullWrRequest : in std_logic;

       Clk           : in std_logic;
       Reset         : in std_logic);
end component;

component PacketReader 
port (

       --Interface with OSAR
       ValidPtrInOsar          : in std_logic;
       PacketPtrInOsar         : in std_logic_vector(c_ChunkPointerWidth-1 downto 0);

       SokOutOsar            : out std_logic;
       SopOutOsar            : out std_logic;
       EopOutOsar            : out std_logic;
       DataOutOsar           : out std_logic_vector(c_DataWidth-1 downto 0);
       NextPacketPtrOutOsar   :  out std_logic_vector(c_ChunkPointerWidth-1 downto 0);
       NextPacketPtrValidOutOsar : out std_logic;


       --Interface with the Sequencer
       RequestValid      : out std_logic;
       Address           : out std_logic_vector(c_AddressWidth-1 downto 0);
       BurstLength       : out std_logic_vector(7 downto 0);
       GiveRequest       : in std_logic;

       --Interface with the FreePointerManager
       Initialization     : in std_logic;
       FreePtr           : out std_logic_vector(c_ChunkPointerWidth-1 downto 0);
       FreePtrValid      : out std_logic;
       WriteError        : in std_logic;

       --Interface with the Sdram Data bus
       SdramGrant        : in std_logic;
       SdramData         : in std_logic_vector(c_DataWidth-1 downto 0);

       Clk               : in std_logic;
       Reset             : in std_logic);
end component;

component FreePtrManager 
port (
       Initialization      : out std_logic;
       --Packet reader Interface
       FreePtrIn           : in std_logic_vector(c_ChunkPointerWidth-1 downto 0);
       FreePtrValidIn      : in std_logic;
       WriteError          : out std_logic;

       --Chunk Writer Interface
       GetFreePtr          : in std_logic;
       FreePtrValidOut     : out std_logic;
       BufferFull          : out std_logic;
       FreePtrOut          : out std_logic_vector(c_ChunkPointerWidth-1 downto 0);

       SdramFreePtrs       : inout std_logic_vector(c_DataWidth-1 downto 0);
       SdramFreePtrFifoAddr: out std_logic_vector(c_AddressWidth-1 downto 0);
       SdramFreePtrOp      : out std_logic;
       SdramFreePtrOpValid : out std_logic;
       SdramFreePtrGrant   : in std_logic;
       SdramFreePtrPullReq : in std_logic;

       Clk                 : in std_logic;
       Reset               : in std_logic);
end component;


component RequestSequencer 
port (
       --From the CHunk Writer
       WrRequestValid          : in std_logic;
       AddressChWr             : in std_logic_vector(c_AddressWidth-1 downto 0);
       BurstLengthChWr         : in std_logic_vector(7 downto 0);
       PullWrRequest           : out std_logic;

       DataFromChunkWriter     : in std_logic_vector(c_DataWidth-1 downto 0);
       GetData                 : out std_logic;

       --From the packet reader
       RdRequestValid          : in std_logic;
       AddressPktRd            : in std_logic_vector(c_AddressWidth-1 downto 0);
       BurstLengthRd       : in std_logic_vector(7 downto 0);
       PullRdRequest           : out std_logic;

       DataToPacketReader      : out std_logic_vector(c_DataWidth-1 downto 0);
       ReceiveData             : out std_logic;

       --From the FreePtrmanager
       SdramFreePtrs           : inout std_logic_vector(c_DataWidth-1 downto 0);
       SdramFreePtrFifoAddr    : in std_logic_vector(c_AddressWidth-1 downto 0);
       SdramFreePtrOp          : in std_logic;
       SdramFreePtrOpValid     : in std_logic;
       SdramFreePtrGrant       : out std_logic;
       SdramFreePtrPullReq     : out std_logic;


       --To the SDRAM
       SdramAddr               : out   std_logic_vector(11 downto 0);
       SdramData               : inout std_logic_vector(c_DataWidth -1 downto 0);
       SdramRas                : out   std_logic;
       SdramCas                : out   std_logic;
       SdramCe                 : out   std_logic_vector(1 downto 0);
       SdramBa                 : out   std_logic_vector(1 downto 0);
       SdramS                  : out   std_logic_vector(1 downto 0);
       SdramWe                 : out   std_logic;

       Clk                     : in std_logic;
       Reset                   : in std_logic);
end component;

       --ChunkWriter-FreePointerManager
signal       GetFreePtrChWr    : std_logic;
signal       FreePtrValidChWr  : std_logic;
signal       BufferFullChWr    : std_logic;
signal       FreePtrChWr       : std_logic_vector(c_ChunkPointerWidth-1 downto 0);

       --PacketReader-FreePointerManager
signal       FreePtrPktRd           : std_logic_vector(c_ChunkPointerWidth-1 downto 0);
signal       FreePtrValidPktRd      : std_logic;
signal       WriteErrorPktRd        : std_logic;


       --ChunkWriter-Request Sequencer
signal       WrRequestValid         : std_logic;
signal       AddressChWr            : std_logic_vector(c_AddressWidth-1 downto 0);
signal       BurstLengthChWr        : std_logic_vector(7 downto 0);
signal       PullWrRequest          : std_logic;

signal       DataFromChunkWriter    : std_logic_vector(c_DataWidth-1 downto 0);
signal       GetData                : std_logic;

       --PacketReader-RequestSequencer
signal       RdRequestValid         : std_logic;
signal       AddressPktRd           : std_logic_vector(c_AddressWidth-1 downto 0);
signal       BurstLengthRd          : std_logic_vector(7 downto 0);
signal       PullRdRequest          : std_logic;

signal       DataToPacketReader     : std_logic_vector(c_DataWidth-1 downto 0);
signal       ReceiveData            : std_logic;

        --FreePtrManager-RequestSequencer
signal       SdramFreePtrs       : std_logic_vector(c_DataWidth-1 downto 0);
signal       SdramFreePtrFifoAddr: std_logic_vector(c_AddressWidth-1 downto 0);
signal       SdramFreePtrOp      : std_logic;
signal       SdramFreePtrOpValid : std_logic;
signal       SdramFreePtrGrant   : std_logic;
signal       SdramFreePtrPullReq : std_logic;

signal       InitializationOn    : std_logic;

signal       ClkSopInIsar         : std_logic;
signal       ClkSokInIsar         : std_logic;
signal       ClkEopInIsar         : std_logic;
signal       ClkPacketPtrInIsar     : std_logic_vector(c_ChunkPointerWidth-1 downto 0);
signal       ClkDataInIsar        : std_logic_vector(c_DataWidth-1 downto 0);

begin

MyRequestSequencer: RequestSequencer
port map(
       --From the CHunk Writer
       WrRequestValid           =>  WrRequestValid, 
       AddressChWr              =>  AddressChWr, 
       BurstLengthChWr          =>  BurstLengthChWr,
       PullWrRequest            =>  PullWrRequest, 

       DataFromChunkWriter      => DataFromChunkWriter, 
       GetData                  => GetData, 

       --From the packet reader
       RdRequestValid           => RdRequestValid, 
       AddressPktRd             => AddressPktRd, 
       BurstLengthRd            => BurstLengthRd,
       PullRdRequest            => PullRdRequest, 

       DataToPacketReader       => DataToPacketReader, 
       ReceiveData              => ReceiveData, 

       --From the FreePtrManager
       SdramFreePtrs            => SdramFreePtrs,
       SdramFreePtrFifoAddr     => SdramFreePtrFifoAddr, 
       SdramFreePtrOp           => SdramFreePtrOp, 
       SdramFreePtrOpValid      => SdramFreePtrOpValid, 
       SdramFreePtrGrant        => SdramFreePtrGrant, 
       SdramFreePtrPullReq      => SdramFreePtrPullReq, 

       --To the SDRAM
       SdramAddr                => SdramAddr, 
       SdramData                => SdramData, 
       SdramRas                 => SdramRas, 
       SdramCas                 => SdramCas, 
       SdramCe                  => SdramCe, 
       SdramBa                  => SdramBa, 
       SdramS                   => SdramS, 
       SdramWe                  => SdramWe, 

       Clk                      => Clk, 
       Reset                    => Reset ); 


process(Clk)
begin
if Clk = '1' and Clk'event then
   if Reset = '0' then
      ClkSopInIsar             <= '0'; 
      ClkSokInIsar             <= '0'; 
      ClkEopInIsar             <= '0'; 
      ClkPacketPtrInIsar     <= (others => '0'); 
      ClkDataInIsar            <= (others => '0'); 
   else
      ClkSopInIsar             <= SopInIsar; 
      ClkSokInIsar             <= SokInIsar; 
      ClkEopInIsar             <= EopInIsar; 
      ClkPacketPtrInIsar       <= PacketPtrInIsar; 
      ClkDataInIsar            <= DataInIsar; 
   end if;
end if;
end process;




MyChunkWriter: ChunkWriter
port map(
       --From the ISAR
       SopInIsar         => ClkSopInIsar, 
       SokInIsar         => ClkSokInIsar, 
       EopInIsar         => ClkEopInIsar, 
       PacketPtrInIsar     => ClkPacketPtrInIsar, 
       DataInIsar        => ClkDataInIsar, 
       NextPacketPtrOutIsar       => NextPacketPtrOutIsar, 
       NextPacketPtrValidOutIsar => NextPacketPtrValidOutIsar, 
       BufferFullOutIsar  => BufferFullOutIsar, 

       --To free pointer manager
       Initialization => InitializationOn,
       GetFreePtr    => GetFreePtrChWr, 
       FreePtrValid  => FreePtrValidChWr, 
       BufferFull    => BufferFullChWr, 
       FreePtr       => FreePtrChWr, 

       --To SDRAM controller
       AddressOut    => AddressChWr, 
       BurstLengthOut  => BurstLengthChWr,
       DataOut       => DataFromChunkWriter, 
       ChunkAvailable=> WrRequestValid, 
       GetData       => GetData, 
       PullWrRequest => PullWrRequest, 

       Clk           => Clk, 
       Reset         => Reset); 


MyFreePtrManager: FreePtrManager
port map(
       Initialization => InitializationOn,
       --Packet reader Interface
       FreePtrIn         => FreePtrPktRd, 
       FreePtrValidIn    => FreePtrValidPktRd, 
       WriteError        => WriteErrorPktRd, 

       --Chunk Writer Interface
       GetFreePtr        => GetFreePtrChWr, 
       FreePtrValidOut   => FreePtrValidChWr, 
       BufferFull        => BufferFullChWr, 
       FreePtrOut        => FreePtrChWr, 

       --request sequencer interface
       SdramFreePtrs            => SdramFreePtrs,
       SdramFreePtrFifoAddr     => SdramFreePtrFifoAddr, 
       SdramFreePtrOp           => SdramFreePtrOp, 
       SdramFreePtrOpValid      => SdramFreePtrOpValid, 
       SdramFreePtrGrant        => SdramFreePtrGrant, 
       SdramFreePtrPullReq      => SdramFreePtrPullReq, 

       Clk           => Clk, 
       Reset         => Reset); 

MyPacketReader: PacketReader
port map(
       ValidPtrInOsar           =>  ValidPtrInOsar, 
       PacketPtrInOsar          => PacketPtrInOsar, 

       SokOutOsar             => SokOutOsar, 
       SopOutOsar             => SopOutOsar, 
       EopOutOsar             => EopOutOsar,
       DataOutOsar            => DataOutOsar, 
       NextPacketPtrOutOsar    =>  NextPacketPtrOutOsar, 
       NextPacketPtrValidOutOsar  => NextPacketPtrValidOutOsar, 


       --Interface with the Sequencer
       RequestValid      => RdRequestValid, 
       Address           => AddressPktRd, 
       BurstLength       => BurstLengthRd,
       GiveRequest       => PullRdRequest, 

       SdramGrant        => ReceiveData, 
       SdramData         => DataToPacketReader, 

       --Interface with the FreePointerManager
       Initialization    => InitializationOn,
       FreePtr           => FreePtrPktRd,
       FreePtrValid      => FreePtrValidPktRd, 
       WriteError        => WriteErrorPktRd, 


       Clk               => Clk, 
       Reset             => Reset); 

Initialization <= InitializationOn;
end struct;

