--  applied research laboratory
--  washington university in st. louis
--
--  file:  rad_loopback_core.vhd
--  top level structure for rad fpga with ingress/egress loopback modules
--  created by:  john w. lockwood (lockwood@arl.wustl.edu),
--  david e. taylor (det3@arl.wustl.edu)
--
library ieee;
use ieee.std_logic_1164.all;

entity rad_loopback_core is
  port (
    -- clocks
    rad_clk  : in std_logic;
    rad_clkb : in std_logic;

    -- reset & reconfig
    rad_reset : in std_logic;
    rad_ready : out std_logic;

    -- ingress path
    -- input
    soc_lc_nid   : in  std_logic;
    d_lc_nid     : in  std_logic_vector(31 downto 0);
    tcaff_lc_rad : out std_logic;
    -- output
    soc_lc_rad   : out std_logic;
    d_lc_rad     : out std_logic_vector(31 downto 0);
    tcaff_lc_nid : in  std_logic;

    -- egress path
    -- input
    soc_sw_nid   : in  std_logic;
    d_sw_nid     : in  std_logic_vector(31 downto 0);
    tcaff_sw_rad : out std_logic;
    -- output
    soc_sw_rad   : out std_logic;
    d_sw_rad     : out std_logic_vector(31 downto 0);
    tcaff_sw_nid : in  std_logic;

    -- SRAM Module 1 (Ingress Side)
    SRAM1_ADDR  : OUT   STD_LOGIC_VECTOR(17 DOWNTO 0);
    SRAM1_DATA  : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0);
    SRAM1_CE    : OUT   STD_LOGIC;      -- Chip Enable: LOW=drive
    SRAM1_GW    : OUT   STD_LOGIC;      -- Global Write: LOW=write
    SRAM1_PIN84 : OUT   STD_LOGIC;

--     -- SRAM Module 2 (Egress side)
--     SRAM2_ADDR  : OUT   STD_LOGIC_VECTOR(17 DOWNTO 0);
--     SRAM2_DATA  : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0);
--     SRAM2_CE    : OUT   STD_LOGIC;      -- Chip Enable: LOW=drive
--     SRAM2_GW    : OUT   STD_LOGIC;      -- Global Write: LOW=write
--     SRAM2_PIN84 : OUT   STD_LOGIC;

    -- SDRAM Module 1 (Ingress side)
    SDR1_ADDR   : OUT   STD_LOGIC_VECTOR(11 DOWNTO 0);
    SDR1_DATA   : INOUT STD_LOGIC_VECTOR(63 DOWNTO 0);
    SDR1_RAS    : OUT   STD_LOGIC;        -- Row Address Strobe
    SDR1_CAS    : OUT   STD_LOGIC;        -- Column Address Strobe
    SDR1_CE     : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  -- Chip Enable
    SDR1_BA     : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  --
    SDR1_S      : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  -- 
    SDR1_SDA    : OUT   STD_LOGIC;        --
    SDR1_SDC    : OUT   STD_LOGIC;        --
    SDR1_WE     : OUT   STD_LOGIC;        -- Write Enable

--     -- SDRAM Module 2 (Egress side)
--     SDR2_ADDR   : OUT   STD_LOGIC_VECTOR(11 DOWNTO 0);
--     SDR2_DATA   : INOUT STD_LOGIC_VECTOR(63 DOWNTO 0);
--     SDR2_RAS    : OUT   STD_LOGIC;        -- Row Address Strobe
--     SDR2_CAS    : OUT   STD_LOGIC;        -- Column Address Strobe
--     SDR2_CE     : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  -- Chip Enable
--     SDR2_BA     : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  --
--     SDR2_S      : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  -- 
--     SDR2_SDA    : OUT   STD_LOGIC;        --
--     SDR2_SDC    : OUT   STD_LOGIC;        --
--     SDR2_WE     : OUT   STD_LOGIC;        -- Write Enable

-- test connector pins
--    rad_test1 : out std_logic_vector(15 downto 0);
--    rad_test2 : out std_logic_vector(15 downto 0);
    -- test led pins
    rad_led1 : out std_logic;
    rad_led2 : out std_logic;
    rad_led3 : out std_logic;
    rad_led4 : out std_logic
    );
end rad_loopback_core;
-------------------------------------------------------------------------------
architecture structure of rad_loopback_core is

-- component declarations

  component loopback_module
    port (
      clk         : in  std_logic;
      reset_l     : in  std_logic;
      soc_mod_in  : in  std_logic;
      d_mod_in    : in  std_logic_vector(31 downto 0);
      tca_mod_in  : out std_logic;
      soc_out_mod : out std_logic;
      d_out_mod   : out std_logic_vector(31 downto 0);
      tca_out_mod : in  std_logic
      );
  end component;

  component app_module
    port (
      clk         : in  std_logic;
      reset_l     : in  std_logic;
      soc_mod_in  : in  std_logic;
      d_mod_in    : in  std_logic_vector(31 downto 0);
      tca_mod_in  : out std_logic;
      soc_out_mod : out std_logic;
      d_out_mod   : out std_logic_vector(31 downto 0);
      tca_out_mod : in  std_logic;
      enable_l    : in  std_logic;
      ready_l     : out std_logic;
      mod1_req   : out    std_logic;
      mod1_gr    : in   std_logic;
      mod1_rw    : out    std_logic;
      mod1_addr  : out    std_logic_vector(18 downto 0);
      mod1_d_in  : in   std_logic_vector(35 downto 0);
      mod1_d_out : out    std_logic_vector(35 downto 0);
      mod2_req   : out    std_logic;
      mod2_gr    : in   std_logic;
      mod2_rw    : out    std_logic;
      mod2_addr  : out   std_logic_vector(18 downto 0);
      mod2_d_in  : in   std_logic_vector(35 downto 0);
      mod2_d_out : out    std_logic_vector(35 downto 0);
      SdramAddr  : out   std_logic_vector(11 downto 0);
      SdramData  : inout std_logic_vector(63 downto 0);
      SdramRas   : out   std_logic;
      SdramCas   : out   std_logic;
      SdramCe    : out   std_logic_vector(1 downto 0);
      SdramBa    : out   std_logic_vector(1 downto 0);
      SdramS     : out   std_logic_vector(1 downto 0);
      SdramWe    : out   std_logic
      );
  end component;

  component sram_interface
    port (
      clkin      : in    std_logic;
      resetin_l  : in    std_logic;
      SRAM_Addr  : out   STD_LOGIC_VECTOR(17 downto 0);
      SRAM_Data  : inout STD_LOGIC_VECTOR(35 downto 0);
      SRAM_gw    : out   STD_LOGIC;
      SRAM_ce    : out   STD_LOGIC;
      SRAM_pin84 : out   STD_LOGIC;
      mod1_req   : in    std_logic;
      mod1_gr    : out   std_logic;
      mod1_rw    : in    std_logic;
      mod1_addr  : in    std_logic_vector(18 downto 0);
      mod1_d_in  : out   std_logic_vector(35 downto 0);
      mod1_d_out : in    std_logic_vector(35 downto 0);
      mod2_req   : in    std_logic;
      mod2_gr    : out   std_logic;
      mod2_rw    : in    std_logic;
      mod2_addr  : in    std_logic_vector(18 downto 0);
      mod2_d_in  : out   std_logic_vector(35 downto 0);
      mod2_d_out : in    std_logic_vector(35 downto 0));
  end component;

  component blink
    port (
      clk1    : in  std_logic;
      clk2    : in  std_logic;
      reset_l : in  std_logic;
      led1    : out std_logic;
      led2    : out std_logic);
  end component;

-- signal declarations

--  signal ingress_test, egress_test : std_logic_vector(31 downto 0);
  signal logic0, logic1 : std_logic;
  signal mod1_req_i   : std_logic;
  signal mod1_gr_i    : std_logic;
  signal mod1_rw_i    : std_logic;
  signal mod1_addr_i  : std_logic_vector(18 downto 0);
  signal mod1_d_in_i  : std_logic_vector(35 downto 0);
  signal mod1_d_out_i : std_logic_vector(35 downto 0);
  signal mod2_req_i   : std_logic;
  signal mod2_gr_i    : std_logic;
  signal mod2_rw_i    : std_logic;
  signal mod2_addr_i  : std_logic_vector(18 downto 0);
  signal mod2_d_in_i  : std_logic_vector(35 downto 0);
  signal mod2_d_out_i : std_logic_vector(35 downto 0);

begin  -- structural

  rad_ready <= not(rad_reset);

-- test pin flops
  test_pin_ff : process (rad_clk)
  begin  -- process test_pin_ff
    if rad_clk'event and rad_clk = '1' then  -- rising clock edge
--        rad_test2 <= ingress_test(31 downto 16);
--        rad_test1 <= ingress_test(15 downto 0);

        rad_led3 <= rad_reset;
        rad_led4 <= not rad_reset;
    end if;
  end process test_pin_ff;

  logic0 <= '0';
  logic1 <= '1';

  sram_int1: sram_interface
    port map (
      clkin      => rad_clk,
      resetin_l  => rad_reset,
      SRAM_Addr  => SRAM1_Addr,
      SRAM_Data  => SRAM1_Data,
      SRAM_gw    => SRAM1_gw,
      SRAM_ce    => SRAM1_ce,
      SRAM_pin84 => SRAM1_pin84,
      mod1_req   => mod1_req_i,
      mod1_gr    => mod1_gr_i,
      mod1_rw    => mod1_rw_i,
      mod1_addr  => mod1_addr_i,
      mod1_d_in  => mod1_d_in_i,
      mod1_d_out => mod1_d_out_i,
      mod2_req   => mod2_req_i,
      mod2_gr    => mod2_gr_i,
      mod2_rw    => mod2_rw_i,
      mod2_addr  => mod2_addr_i,
      mod2_d_in  => mod2_d_in_i,
      mod2_d_out => mod2_d_out_i);
  ingress : app_module 
    port map (
      clk         => rad_clk,
      reset_l     => rad_reset,
      soc_mod_in  => soc_lc_nid,
      d_mod_in    => d_lc_nid,
      tca_mod_in  => tcaff_lc_rad,
      soc_out_mod => soc_lc_rad,
      d_out_mod   => d_lc_rad,
      tca_out_mod => tcaff_lc_nid,
      enable_l    => logic0,
      ready_l     => open,
      mod1_req   => mod1_req_i,
      mod1_gr    => mod1_gr_i,
      mod1_rw    => mod1_rw_i,
      mod1_addr  => mod1_addr_i,
      mod1_d_in  => mod1_d_in_i,
      mod1_d_out => mod1_d_out_i,
      mod2_req   => mod2_req_i,
      mod2_gr    => mod2_gr_i,
      mod2_rw    => mod2_rw_i,
      mod2_addr  => mod2_addr_i,
      mod2_d_in  => mod2_d_in_i,
      mod2_d_out => mod2_d_out_i,
      SdramAddr  => SDR1_ADDR,
      SdramData  => SDR1_DATA,  
      SdramRas   => SDR1_RAS,
      SdramCas   => SDR1_CAS,
      SdramCe    => SDR1_CE,
      SdramBa    => SDR1_BA,         
      SdramS     => SDR1_S,
      SdramWe    => SDR1_WE);

  egress : loopback_module
    port map (
      clk         => rad_clkb,
      reset_l     => rad_reset,
      soc_mod_in  => soc_sw_nid,
      d_mod_in    => d_sw_nid,
      tca_mod_in  => tcaff_sw_rad,
      soc_out_mod => soc_sw_rad,
      d_out_mod   => d_sw_rad,
      tca_out_mod => tcaff_sw_nid);

  blink1 : blink
    port map (
      clk1    => rad_clk,
      clk2    => rad_clkb,
      reset_l => rad_reset,
      led1    => rad_led1,
      led2    => rad_led2);

end structure;







