--  Applied Research Laboratory
--  Washington University in St. Louis
--
--  File:  sram_interface.vhd
--  Interface to Micron ZBT Pipelined SRAM
--  Created by:  David E. Taylor (det3@arl.wustl.edu)
--  Created on:  October 29, 1999
--  Last modified:  December 16, 2000 @ 1:40pm
--
--  VHDL Entity:
--  The following VHDL code implements an interface to a
--  Micron ZBT Pipelined SRAM 512K x 36.  All boundary flops are written
--  to fall in an IOB for a Xilinx Virtex FPGA.
--  READS:  Data will be available 4 clock cycles after address
--  lines are valid.
--  WRITES:  Data, Address, and R/W signal are issued to the interface
--  on the same clock cycle.  Data will be internally delayed two clock
--  cycles to meet the ZBT timing requirements.
--
--  Note that 19 address bits are available for upgrade to 2Mb SRAM.
--  Address bit (18) should is tied to SRAM_PIN84.
--
--  Clock to output delays are provided on IOB flip flops for proper
--  simulation with Micron SRAM model.  They are ignored for synthesis.
--
-------------------------------------------------------------------------------
-- Technology License for Research and Education Purposes
-- 
-- Washington University ("University") in St. Louis has 
-- developed and is the owner of the copyright for Field-programmable 
-- Port Extender (FPX) Technology.
-- 
-- The University grants to select not-for-profit educational institution 
-- ("Licensee") receiving the Technology the right to use its Technology 
-- solely for Licensee's internal nonprofit research and academic purposes.  
-- This License is nontransferrable, royalty free and non-exclusive.
-- the licensee may reproduce Technology only for educational, 
-- research, backup or archival purposes
-- 
-- UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATIONS, EITHER EXPRESS 
-- OR IMPLIED, FOR THE DESIGNS AND/OR DOCUMENTATION PROVIDED UNDER THIS 
-- AGREEMENT, INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY 
-- AND/OR WARRANTY OF FITNESS FOR A PARTICULAR USE AND/OR WARRANTY OF FREEDOM 
-- OF INFRINGEMENT FROM THIRD PARTY PROPRIETARY RIGHTS.  
-- 
-- Licensee assumes the entire risk as to the performance of Technology and/or 
-- associated materials, and to the performance and validity of information 
-- generated using Technology.  
--
-- For complete terms of the agreement, review FPX-license.txt,
-- which is included in this file distribution.
--
-- 
-- Washington University
-- Applied Research Laboratory
-- 1 Brookings Drive
-- Campus Box 1045
-- Saint Louis, MO 63130  USA
-- Phone: (314) 935-4460
-- Web: http://www.arl.wustl.edu/arl/projects/fpx/
------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_UNSIGNED.all;
-- synthesis translate_off
library unisim;
-- synthesis translate_on

ENTITY sram_interface IS
  PORT(clkin        : in    std_logic ;
       resetin_l    : in    std_logic ;
       -- SRAM Module I/O               -- 
       SRAM_Addr    : out   STD_LOGIC_VECTOR(17 downto 0);  -- 18 Address bits 
       SRAM_Data    : inout STD_LOGIC_VECTOR(35 downto 0);
       SRAM_gw      : out   STD_LOGIC;  -- Global Write: LOW=write
       SRAM_ce      : out   STD_LOGIC;  -- Chip Enable: LOW=drive
       SRAM_pin84   : out   STD_LOGIC;  -- Address MSB; unused in 1Mb part
       -- Module 1 Interface
       mod1_req     : in    std_logic ;
       mod1_gr      : out   std_logic ;
       mod1_rw      : in    std_logic ;
       mod1_addr    : in    std_logic_vector(18 downto 0) ;
       mod1_d_in    : out   std_logic_vector(35 downto 0) ;
       mod1_d_out   : in    std_logic_vector(35 downto 0) ;
       -- Module 2 Interface
       mod2_req     : in    std_logic ;
       mod2_gr      : out   std_logic ;
       mod2_rw      : in    std_logic ;
       mod2_addr    : in    std_logic_vector(18 downto 0) ;
       mod2_d_in    : out   std_logic_vector(35 downto 0) ;
       mod2_d_out   : in    std_logic_vector(35 downto 0));

END sram_interface ;
--
ARCHITECTURE behavioral OF sram_interface IS
--
-- Component Declarations
--
  COMPONENT IOBUF_F_12
    port ( 
      O  : out std_ulogic; 
      I  : in  std_ulogic;
      IO : inout std_logic;
      T  : in std_logic
      );
  end COMPONENT;

  attribute syn_black_box : boolean;
  attribute syn_black_box of IOBUF_F_12: component is true;

--
-- synthesis translate_off
  for all : IOBUF_F_12 use entity unisim.IOBUF_F_12(IOBUF_F_12_V);
-- synthesis translate_on
--
-- Signal declarations
--
  type states is (IDLE, MOD1_ACCESS, MOD1_HOLD, MOD2_ACCESS, MOD2_HOLD) ;
  signal   state                        : states ;
  signal   nx_state                     : states ;
--
  signal   data_in                      : std_logic_vector(35 downto 0) ;
  signal   data_out                     : std_logic_vector(35 downto 0) ;
  signal   addr                         : std_logic_vector(18 downto 0) ;
  signal   addr_sel                     : std_logic ;
  signal   data_ff_in                   : std_logic_vector(35 downto 0) ;
  signal   data_ff_out                  : std_logic_vector(35 downto 0) ;
  signal   data_out_delay1              : std_logic_vector(35 downto 0) ;
  signal   data_out_delay2              : std_logic_vector(35 downto 0) ;
  signal   read_write                   : std_logic ;  -- (1 = READ, 0 = WRITE)
  signal   data_tri_state_enable        : std_logic_vector(35 downto 0) ;  -- (1 = READ, 0 = WRITE)
  signal   data_dir                     : std_logic_vector(35 downto 0) ;  -- (1 = READ, 0 = WRITE)
  signal   data_tri_state_enable_delay1 : std_logic ;  -- (1 = READ, 0 = WRITE)
  signal   data_tri_state_enable_delay2 : std_logic ;  -- (1 = READ, 0 = WRITE)
  signal   mem_enable                   : std_logic ;  -- (1 = enable, 0 = disable)
  signal   mem_enable_delay             : std_logic ;  -- (1 = enable, 0 = disable)
  signal   next_ce                      : std_logic ;
--
  constant pad_delay                        : time := 2 ns;
--
BEGIN
--
-- Initialization state assignments/transitions
--
  CLKD                                  : process (clkin)
  begin
    if (clkin = '1' and clkin'event) then
      if (resetin_l = '0') then
        state <= IDLE ;
      else
        state <= nx_state ;
      end if;
    end if;
  end process ;
--
-- Next State assignments
--
  STATE_TRANS : process (state, mod1_req, mod2_req)
  begin
    nx_state <= state ;
    case state is
      when IDLE => if (mod1_req = '1') then
                     nx_state <= MOD1_ACCESS ;
                   elsif (mod2_req = '1') then
                     nx_state <= MOD2_ACCESS ;
                   else
                     nx_state <= IDLE ;
                   end if;
      when MOD1_ACCESS => if (mod1_req = '0' and mod2_req = '0') then
                            nx_state <= IDLE ;
                          elsif (mod1_req = '0' and mod2_req = '1') then
                            nx_state <= MOD2_ACCESS ;
                          elsif (mod1_req = '1' and mod2_req = '1') then
                            nx_state <= MOD1_HOLD ;
                          else
                            nx_state <= MOD1_ACCESS ;
                          end if;
      when MOD2_ACCESS => if (mod1_req = '0' and mod2_req = '0') then
                            nx_state <= IDLE ;
                          elsif (mod1_req = '1' and mod2_req = '0') then
                            nx_state <= MOD1_ACCESS ;
                          elsif (mod1_req = '1' and mod2_req = '1') then
                            nx_state <= MOD2_HOLD ;
                          else
                            nx_state <= MOD2_ACCESS ;
                          end if;
      when MOD1_HOLD => if (mod1_req = '0' and mod2_req = '0') then
                          nx_state   <= IDLE ;
                        elsif (mod1_req = '0' and mod2_req = '1') then
                          nx_state   <= MOD2_ACCESS ;
                        else
                          nx_state   <= MOD1_HOLD ;
                        end if;
      when MOD2_HOLD => if (mod1_req = '0' and mod2_req = '0') then
                          nx_state   <= IDLE ;
                        elsif (mod1_req = '1' and mod2_req = '0') then
                          nx_state   <= MOD1_ACCESS ;
                        else
                          nx_state   <= MOD2_HOLD ;
                        end if;
    end case ;
  end process ;
--
-- C/L for grant signals.
--
  mod1_gr <= '1' when (state=MOD1_ACCESS) else '0';
  mod2_gr <= '1' when (state=MOD2_ACCESS) else '0';
--
-- C/L for read/write and enable signals to memory.
--
  read_write <= mod1_rw when (state=MOD1_ACCESS or state=MOD1_HOLD) else
                mod2_rw when (state = MOD2_ACCESS or state = MOD2_HOLD) else
                '1';
--
-- Signal for chip and flop enables.
--
  mem_enable <= '1' when (state=IDLE) else '0';
--
-- Delayed signal for completion of write commands
--  
  mem_enable_delay_proc : process (clkin)
  begin  -- process mem_enable_delay
    if clkin'event and clkin = '1' then  -- rising clock edge
      mem_enable_delay <= mem_enable;
    end if;
  end process mem_enable_delay_proc;
--
-- Chip enable to SRAM
--  
  next_ce <= '0' when (mem_enable='0' or mem_enable_delay='0') else '1';
--
-- Data demultiplexor
--
  mod1_d_in(35 downto 0) <= data_ff_in(35 downto 0);
  mod2_d_in(35 downto 0) <= data_ff_in(35 downto 0);
--
-- Data multiplexor
--
  data_out(35 downto 0) <= mod2_d_out(35 downto 0) when (state = MOD2_ACCESS or state = MOD2_HOLD) else
                           mod1_d_out(35 downto 0);
--
-- Write Data Delay Flops
-- * For ZBT SRAM, write data must appear two clock cycles after address.
--
  WRITE_DATA_DELAY_FLOP:process(clkin)
  BEGIN
    if (clkin'event and clkin = '1') then
      if (next_ce = '0') then
        data_out_delay2(35 downto 0) <= data_out_delay1(35 downto 0) ;
        data_out_delay1(35 downto 0) <= data_out(35 downto 0)        ;
      end if;
    end if;
  end process WRITE_DATA_DELAY_FLOP ;
--
-- Data Tri-State Buffer Enable logic
--
  TRI_STATE_EN_DELAY_FLOP:process(clkin)
  BEGIN
    if (clkin'event and clkin = '1') then
      if (next_ce = '0') then
        data_tri_state_enable_delay2   <= data_tri_state_enable_delay1 ;
        data_tri_state_enable_delay1   <= read_write                   ;
      end if;
    end if;
  end process TRI_STATE_EN_DELAY_FLOP ;
--
  TRI_STATE_DIR : for i in 35 downto 0 generate
    data_dir(i) <= data_tri_state_enable_delay2;
  end generate TRI_STATE_DIR;
--
-- IOB Data Flops
--
  DATA_BUF_FLOPS:process(clkin)
  BEGIN
    if (clkin'event and clkin = '1') then
      if (next_ce='0') then
        data_tri_state_enable(35 downto 0) <= data_dir(35 downto 0) after pad_delay ;
        data_ff_out(35 downto 0) <= data_out_delay2(35 downto 0) after pad_delay ;
        data_ff_in(35 downto 0)  <= data_in(35 downto 0) after pad_delay         ;
      end if;
    end if;
  end process DATA_BUF_FLOPS ;
--
-- Instantiate the SRAM bi-directional data buffers in IOBs
--
  ALL_SRAM_DATA_PAD:
  for i in 35 downto 0 generate
    SRAM_DATA_PAD : 
      IOBUF_F_12 
        port map ( 
          O  => data_in(i) , 
          I  => data_ff_out(i) ,
          IO => SRAM_Data(i),
          T  => data_tri_state_enable(i)
          );
  end generate ALL_SRAM_DATA_PAD;
--
-- Address multiplexor
--
  with addr_sel select
    addr(18 downto 0) <=
    mod1_addr(18 downto 0) when '0',
    mod2_addr(18 downto 0) when others;

  addr_sel_proc: process (clkin)
  begin  -- process addr_sel_proc
    if clkin'event and clkin = '1' then  -- rising clock edge
      if (state = MOD2_ACCESS or state = MOD2_HOLD) then
        addr_sel <= '1';
      else
        addr_sel <= '0';
      end if;
    end if;
  end process addr_sel_proc;

--  addr(18 downto 0) <= mod1_addr(18 downto 0) when (state=MOD1_ACCESS or state=MOD1_HOLD)     else
--                       mod2_addr(18 downto 0) when (state = MOD2_ACCESS or state = MOD2_HOLD) else
--                       (others => '0');
--
-- IOB Address line flops.
--
  ADDR_FLOP:process(clkin)
  BEGIN
    if (clkin'event and clkin = '1') then
      if (next_ce='0') then
        SRAM_Addr(17 downto 0) <= addr(17 downto 0) after pad_delay;
        SRAM_pin84 <= addr(18) after pad_delay;
      end if;
    end if;
  end process ADDR_FLOP ;
--
-- IOB Global Write signal flop.
--
  GW_FLOP:process(clkin)
  BEGIN
    if (clkin'event and clkin = '1') then
      if (next_ce='0') then
        SRAM_gw <= read_write after pad_delay;
      end if;
    end if;
  end process GW_FLOP ;
--
-- IOB Chip Enable signal flop.
--
  CE_FLOP:process(clkin)
  BEGIN
    if (clkin'event and clkin = '1') then
      SRAM_ce <= next_ce after pad_delay;
    end if;
  end process CE_FLOP ;
--
end behavioral;






